@@ -54,8 +54,8 @@ class Mouse(InfoBaseClass):
     @property
     def project(self) -> str | None:
         "Project associated with the mouse."
-        return self.lims.get("project_name")
-    
+        return self.lims.project_name if self.lims else None
+
     @cached_property
     def state(self) -> State:
         return State(self.id)
@@ -94,9 +94,10 @@ class Projects(enum.Enum):
         "DynamicRoutingTask1Production",
     )
     VB = ("NeuropixelVisualBehavior",)
-
+    TTN = ('TaskTrainedNetworksNeuropixel',)
 
 class Project(InfoBaseClass):
+    
     def __init__(self, lims_project_name: str):
         self.id = str(lims_project_name)
 
