@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import abc
 import contextlib
+import copy
 import csv
 import datetime
 import functools
@@ -10,7 +11,9 @@ import pathlib
 import re
 import tempfile
 import time
-from typing import Any, ClassVar, ForwardRef, Generator, Mapping, Optional, Sequence, Union, Dict, List
+from typing import (Any, ClassVar, Dict, ForwardRef, Generator, Iterable, List, Mapping,
+                    Optional, Sequence, Union)
+
 import np_config
 import np_logging
 import pydantic
@@ -68,26 +71,26 @@ class PlatformJson(pydantic.BaseModel):
     """Writes D1 platform json for lims upload. Just requires a path (dir or dir+filename)."""
 
     # ------------------------------------------------------------------------------------- #
-    # required kwargs on init (any property without a default value or leading underscore): 
+    # required kwargs on init (any property without a default value or leading underscore)
     
     path: pathlib.Path
     "Typically the storage directory for the session. Will be modified on assignment."
     
     # ------------------------------------------------------------------------------------- #
     
-    write_on_update: bool = True
+    file_sync: bool = True
     
     @contextlib.contextmanager
-    def write_disabled(self)  -> Generator[None, None, None]:
+    def sync_disabled(self)  -> Generator[None, None, None]:
         "Context manager to temporarily disable writing to file when a property is updated."
-        self.write_on_update = False
+        self.file_sync = False
         yield
-        self.write_on_update = True
+        self.file_sync = True
         
     class Config:
         validate_assignment = True # coerce types on assignment
         extra = 'allow' # 'forbid' = properties must be defined in the model
-        fields = {'path': {'exclude': True}, 'write_on_update': {'exclude': True}}
+        fields = {'path': {'exclude': True}, 'file_sync': {'exclude': True}}
         arbitrary_types_allowed = True
         
     suffix: ClassVar[str] = "_platformD1.json"
@@ -100,25 +103,42 @@ class PlatformJson(pydantic.BaseModel):
         return self.path.as_posix()
     
     def load_from_existing(self) -> None:
-        "Reads existing file and loads all non-empty fields to self."
-        with self.write_disabled():
-            if self.path.exists():
-                contents = json.loads(self.path.read_text() or "{}")
-                for k, v in contents.items():
-                    if v and v != getattr(self, k, None):
-                        setattr(self, k, v)
+        "Update empty fields with non-empty fields from file."
+        with self.sync_disabled():
+            if not self.path.exists():
+                return
+            contents = json.loads(self.path.read_text() or "{}")
+            for k, v in contents.items():
+                if not v:
+                    continue
+                if isinstance(v, (dict, list)) and not all(_ for _ in v):
+                    continue
+                setattr(self, k, v)
     
     def __setattr__(self, name, value):
+        
+        # if field is in non-validated list, just set it 
+        if name in (k for k, v in self.Config.fields.items() if v.get('exclude')):
+            return super().__setattr__(name, value)
+        
+        if self.file_sync:
+            # fetch fields from disk before writing, in case another process updated the
+            # file since we last read it
+            self.load_from_existing()
+            
         _ = super().__setattr__(name, value)
-        is_in_json = name not in (k for k, v in self.Config.fields.items() if v.get('exclude'))
-        if self.write_on_update and is_in_json:
+        
+        logger.info("Updated %s.%s = %s", self.path.name, name, value)
+            
+        if self.file_sync:
             self.write()
+            
         return _
     
     def write(self): 
         self.path.parent.mkdir(parents=True, exist_ok=True)
         self.path.touch()
-        with self.write_disabled():
+        with self.sync_disabled():
             self.platform_json_save_time = np_config.normalize_time(time.time())
         self.path.write_text(self.json(indent=4))
         logger.debug("%s wrote to %s", self.__class__.__name__, self.path.as_posix())
@@ -184,8 +204,10 @@ class PlatformJson(pydantic.BaseModel):
     InsertionNotes: Dict[str, Dict] = pydantic.Field(default_factory=dict)
     ExperimentStartTime: Union[PlatformJsonDateTime, str] = ''
     stimulus_name: str = ""
+    "MTrain stage (?)."
     script_name: Union[pathlib.Path, str] = ""
-
+    "Path to stimulus script."
+    
     # post-experiment ---------------------------------------------------------------------- #
     ExperimentCompleteTime: Union[PlatformJsonDateTime, str] = ''
     ExperimentNotes: Dict[str, Dict[str, Any]] = dict(
@@ -203,19 +225,25 @@ class PlatformJson(pydantic.BaseModel):
     
     manipulator_coordinates: Dict[str, Dict[Any, Any]] = pydantic.Field(default_factory=dict)
 
-    files: Dict[str, Dict[str, str]] = pydantic.Field(default_factory=dict)
+    files: Dict[str, Dict[str, Dict[str, List]]] = pydantic.Field(default_factory=dict)
 
     def update(self, field, new) -> None:
+        self.load_from_existing()
         existing = getattr(self, field)
         if (not new and new is not False) or existing == new:
             return
+        
+        # now merge the new value with the existing value in the file if it's a dict
+        with contextlib.suppress(TypeError, AttributeError):
+            new = np_config.merge(copy.deepcopy(getattr(self, field)), new)
+            
         print('Updating %s %s: %s -> %s', self.path.name, field, existing, new)
         setattr(self, field, new)
     
 def update_from_session(pj: PlatformJson, session) -> None:
     "Updates fields in a platform json file."
     
-    with pj.write_disabled():
+    with pj.sync_disabled():
         pj.update('operatorID', str(session.user)) # don't need to convert here, `update` will compare values with existing
         pj.update('sessionID', str(session.id))
         pj.update('mouseID', str(session.mouse.id))
