@@ -5,6 +5,7 @@ import copy
 import datetime
 import doctest
 import functools
+import itertools
 import os
 import pathlib
 from typing import Any, Callable, Generator, Iterable, Optional, Union
@@ -215,9 +216,9 @@ class Session:
         
         Note that habs are classed as ecephys sessions: use `is_hab`.
         """
-        if not self.lims or not self.lims.get("storage_directory"):
+        if not self.lims:
             return None
-        return "ecephys_session" in self.lims["storage_directory"]
+        return "ecephys_session" in self.lims.get("storage_directory", "")
 
     @property
     def is_hab(self) -> bool | None:
@@ -443,7 +444,7 @@ class Session:
         pj = path_or_obj
         # TODO get files dict, fetch files
     
-    @property
+    @cached_property
     def experiment_start(self) -> datetime.datetime:
         """Start time estimated from platform.json, for finding files created during
         experiment. Not relevant for D2 files.
@@ -459,7 +460,7 @@ class Session:
         logger.warning('Could not find experiment start time in %s: using start of day instead', self.platform_json)
         return datetime.datetime(*self.date.timetuple()[:5])
     
-    @property
+    @cached_property
     def experiment_end(self) -> datetime.datetime:
         """End time estimated from platform.json, for finding files created during
         experiment. Not relevant for D2 files.
@@ -578,9 +579,30 @@ def sessions(
             
         yield session
 
-
+def cleanup_npexp():
+    """Remove empty dirs, 366122 dirs, move habs"""
+    for _ in itertools.chain(NPEXP_ROOT.iterdir(), (NPEXP_ROOT / "habituation").iterdir()):
+        if not _.is_dir():
+            continue
+        if not any(_.iterdir()):
+            _.rmdir()
+            logger.info("Removed empty dir %s", _.name)
+            continue
+        try:
+            session = Session(_)
+        except SessionError:
+            continue
+        if session.is_hab and _.parent == NPEXP_ROOT:
+            try:
+                _.replace(NPEXP_ROOT / "habituation" / "backup" / _.name)
+            except OSError as exc:
+                logger.error("Moving hab failed: %r", exc)
+            else:
+                logger.info("Moved %s to habituation/backup", _.name)
+                
 if __name__ == "__main__":
-
+    logger = np_logging.getLogger()
+    cleanup_npexp()
     if is_connected("lims2"):
         doctest.testmod(verbose=True)
         # optionflags=(doctest.ELLIPSIS, doctest.NORMALIZE_WHITESPACE,
