@@ -74,7 +74,7 @@ class Host(enum.Enum):
 
 def is_connected(host: str | Host) -> bool:
     "Use OS's `ping` cmd to check if `host` is connected."
-    command = ["ping", "-n" if "win" in sys.platform else "-c", "1", host]
+    command = ["ping", "-n" if "win" in sys.platform else "-c", "1", str(host)]
     try:
         return subprocess.call(command, stdout=subprocess.PIPE, timeout=0.1) == 0
     except subprocess.TimeoutExpired:
@@ -103,7 +103,6 @@ def is_valid_session_id(session_id: int | str) -> bool:
         return False
     return bool(LIMS2SessionInfo(session_id))
 
-
 def lims_session_id(path: PathLike) -> str | None:
     """
     Get valid lims ecephys/behavior session id from str or path.
